;nyquist plug-in
;version 4
;type tool
;name "Mute"
;author "Steve Daulton"
;release 3.0.0
;copyright "Released under terms of the GNU General Public License version 2"

(defun get-env-level (track-id at-time)
  ;; Find the envelope level at 'at-time' in track 'id'
  ;; track-id is zero indexed.
  (let ((info (aud-get-info "Envelopes"))
        pointlist track points time val start-time start-val)
    (dolist (aclip info)
      (setf track (assoc 'track aclip))
      (setf points (assoc 'points aclip))
      (when (and (= (second track) track-id)
                 points)
        (setf pointlist (second points))
        (dolist (duple pointlist)
          (setf time (second (first duple)))
          (setf val (second (second duple)))
          (if (< time at-time)
              (psetq start-time time start-val val)
              (return)))))
      (log-interpolate start-time start-val time val at-time)))

(defun log-interpolate (t0 v0 t1 v1 at-time)
  (cond
    ((not t1) 1.0)  ;No envelope points.
    ((not t0) v1)   ;No envelope points before 'at-time'.
    ((= t0 t1) v1)  ;No envelope points after 'at-time'.
    (t  ;'at-time' between t0 and t1.
      (let* ((dur (- t1 t0))
             (v0db (linear-to-db v0))
             (v1db (linear-to-db v1))
             (ratio (/ (- at-time t0) dur)))
        (db-to-linear (+ v0db (* (- v1db v0db) ratio)))))))

(defun mute (t0 v0 t1 v1)
  (when (< (- t1 t0) 0.02)
    (throw 'err "Error.\nSelection too short"))
  (aud-do-command "SetEnvelope" :time t0 :value v0)
  (aud-do-command "SetEnvelope" :time (+ t0 0.01) :value 0)
  (aud-do-command "SetEnvelope" :time (- t1 0.01) :value 0)
  (aud-do-command "SetEnvelope" :time t1 :value v1))

(defun process ()
  (let ((tracks (get '*selection* 'tracks))
        (sel-start (get '*selection* 'start))
        (sel-end (get '*selection* 'end)))
    (dolist (track-num tracks)
      (setf track-num (1- track-num))
      ;; TODO: It would be more efficient to get start and end levels
      ;; at the same time, to avoid looping through envelope points twice.
      ;; We could then also check there are no envelope points between.
      (setf start-level (get-env-level track-num sel-start))
      (setf end-level (get-env-level track-num sel-end))
      (mute sel-start start-level sel-end end-level))
    ""))

(catch 'err (process))
